@@ -59,7 +59,7 @@ add_s.weights <- function(m,
                           s.weights = NULL,
                           data = NULL) {
 
-  if (!inherits(m, "matchit")) stop("'m' must be a matchit object, the output of a call to matchit().", call. = FALSE)
+  chk::chk_is(m, "matchit")
 
   if (!is.null(s.weights)) {
     if (!is.numeric(s.weights)) {
@@ -70,43 +70,45 @@ add_s.weights <- function(m,
           env <- parent.frame()
         }
         data <- eval(m$call$data, envir = env)
-        if (length(data) == 0) stop("A dataset could not be found. Please supply an argument to 'data' containing the original dataset used in the matching.", call. = FALSE)
+        if (length(data) == 0) {
+          .err("a dataset could not be found. Please supply an argument to `data` containing the original dataset used in the matching")
+        }
       }
       else {
         if (!is.data.frame(data)) {
           if (is.matrix(data)) data <- as.data.frame.matrix(data)
-          else stop("'data' must be a data frame.", call. = FALSE)
+          else .err("`data` must be a data frame")
         }
         if (nrow(data) != length(m$treat)) {
-          stop("'data' must have as many rows as there were units in the original call to matchit().", call. = FALSE)
+          .err("`data` must have as many rows as there were units in the original call to `matchit()`")
         }
       }
 
       if (is.character(s.weights)) {
         if (is.null(data) || !is.data.frame(data)) {
-          stop("If 's.weights' is specified a string, a data frame containing the named variable must be supplied to 'data'.", call. = FALSE)
+          .err("if `s.weights` is specified a string, a data frame containing the named variable must be supplied to `data`")
         }
         if (!all(s.weights %in% names(data))) {
-          stop("The name supplied to 's.weights' must be a variable in 'data'.", call. = FALSE)
+          .err("the name supplied to `s.weights` must be a variable in `data`")
         }
         s.weights.form <- reformulate(s.weights)
         s.weights <- model.frame(s.weights.form, data, na.action = "na.pass")
-        if (ncol(s.weights) != 1) stop("'s.weights' can only contain one named variable.", call. = FALSE)
+        if (ncol(s.weights) != 1) .err("`s.weights` can only contain one named variable")
         s.weights <- s.weights[[1]]
       }
-      else if (inherits(s.weights, "formula")) {
-        s.weights.form <- update(s.weights, NULL ~ .)
+      else if (rlang::is_formula(s.weights)) {
+        s.weights.form <- update(terms(s.weights, data = data), NULL ~ .)
         s.weights <- model.frame(s.weights.form, data, na.action = "na.pass")
-        if (ncol(s.weights) != 1) stop("'s.weights' can only contain one named variable.", call. = FALSE)
+        if (ncol(s.weights) != 1) .err("`s.weights` can only contain one named variable")
         s.weights <- s.weights[[1]]
       }
       else {
-        stop("'s.weights' must be supplied as a numeric vector, string, or one-sided formula.", call. = FALSE)
+        .err("`s.weights` must be supplied as a numeric vector, string, or one-sided formula")
       }
     }
 
-    if (anyNA(s.weights)) stop("Missing values are not allowed in 's.weights'.", call. = FALSE)
-    if (length(s.weights) != length(m$treat)) stop("'s.weights' must be the same length as the treatment vector.", call. = FALSE)
+    chk::chk_not_any_na(s.weights)
+    if (length(s.weights) != length(m$treat)) .err("`s.weights` must be the same length as the treatment vector")
 
     names(s.weights) <- names(m$treat)
 
@@ -117,5 +119,5 @@ add_s.weights <- function(m,
     m$nn <- nn(m$treat, m$weights, m$discarded, s.weights)
   }
 
-  return(m)
+  m
 }
