@@ -181,17 +181,17 @@ qqsum <- function(x, t, w = NULL, standardize = FALSE) {
     t1 <- t == t[1]
     #For binary variables, just difference in means
     ediff <- abs(wm(x[t1], w[t1]) - wm(x[-t1], w[-t1]))
-    return(c(meandiff = ediff, meddiff = ediff, maxdiff = ediff))
+    return(c(meandiff = ediff, maxdiff = ediff))
   }
 
-  for (i in unique(t, nmax = 2)) w[t==i] <- w[t==i]/sum(w[t==i])
+  w <- .make_sum_to_1(w, by = t)
 
   ord <- order(x)
   x_ord <- x[ord]
   w_ord <- w[ord]
   t_ord <- t[ord]
 
-  t1 <- which(t_ord==t_ord[1])
+  t1 <- which(t_ord == t_ord[1])
 
   if (standardize) {
     #Difference between ecdf of x for each group
@@ -205,39 +205,43 @@ qqsum <- function(x, t, w = NULL, standardize = FALSE) {
 
     u <- unique(x_ord)
 
-    wn1 <- sum(w[t1] > 0)
-    wn0 <- sum(w[-t1] > 0)
-
     w1 <- w_ord[t1]
     w0 <- w_ord[-t1]
 
     x1 <- x_ord[t1][w1 > 0]
     x0 <- x_ord[-t1][w0 > 0]
 
+    w1 <- w1[w1 > 0]
+    w0 <- w0[w0 > 0]
+
+    wn1 <- length(w1)
+    wn0 <- length(w0)
+
     if (wn1 < wn0) {
       if (length(u) <= 5) {
-        x0probs <- vapply(u, function(u_) wm(x0 == u_, w0[w0 > 0]), numeric(1L))
-        x0cumprobs <- c(0, cumsum(x0probs)[-length(u)], 1)
-        x0 <- u[findInterval(cumsum(w1[w1 > 0]), x0cumprobs, rightmost.closed = TRUE)]
+        x0probs <- vapply(u, function(u_) wm(x0 == u_, w0), numeric(1L))
+        x0cumprobs <- c(0, .cumsum_prob(x0probs))
+        x0 <- u[findInterval(.cumsum_prob(w1), x0cumprobs, rightmost.closed = TRUE)]
       }
       else {
-        x0 <- approx(cumsum(w0[w0 > 0]), y = x0, xout = cumsum(w1[w1 > 0]), rule = 2,
+        x0 <- approx(.cumsum_prob(w0), y = x0, xout = .cumsum_prob(w1), rule = 2,
                      method = "constant", ties = "ordered")$y
       }
     }
-    else {
+    else if (wn1 > wn0) {
       if (length(u) <= 5) {
-        x1probs <- vapply(u, function(u_) wm(x1 == u_, w1[w1 > 0]), numeric(1L))
-        x1cumprobs <- c(0, cumsum(x1probs)[-length(u)], 1)
-        x1 <- u[findInterval(cumsum(w0[w0 > 0]), x1cumprobs, rightmost.closed = TRUE)]
+        x1probs <- vapply(u, function(u_) wm(x1 == u_, w1), numeric(1L))
+        x1cumprobs <- c(0, .cumsum_prob(x1probs))
+        x1 <- u[findInterval(.cumsum_prob(w0), x1cumprobs, rightmost.closed = TRUE)]
       }
       else {
-        x1 <- approx(cumsum(w1[w1 > 0]), y = x1, xout = cumsum(w0[w0 > 0]), rule = 2,
+        x1 <- approx(.cumsum_prob(w1), y = x1, xout = .cumsum_prob(w0), rule = 2,
                      method = "constant", ties = "ordered")$y
       }
     }
     ediff <- abs(x1 - x0)
   }
-  return(c(meandiff = mean(ediff), maxdiff = max(ediff)))
+
+  c(meandiff = mean(ediff), maxdiff = max(ediff))
 
 }
\ No newline at end of file
