@@ -109,15 +109,7 @@
 #' individual methods pages for information on whether and how this argument is
 #' used. Default is `FALSE` for matching without replacement.
 #' @param m.order for methods that allow it, the order that the matching takes
-#' place. Allowable options depend on the matching method but include
-#' `"largest"`, where matching takes place in descending order of distance
-#' measures; `"smallest"`, where matching takes place in ascending order
-#' of distance measures; `"random"`, where matching takes place in a
-#' random order; and `"data"` where matching takes place based on the
-#' order of units in the data. When `m.order = "random"`, results may
-#' differ across different runs of the same code unless a seed is set and
-#' specified with [set.seed()]. See the individual methods pages for
-#' information on whether and how this argument is used. The default of
+#' place. Allowable options depend on the matching method. The default of
 #' `NULL` corresponds to `"largest"` when a propensity score is
 #' estimated or supplied as a vector and `"data"` otherwise.
 #' @param caliper for methods that allow it, the width(s) of the caliper(s) to
@@ -230,14 +222,14 @@
 #' (in the case of k:1 matching) or the stratum they belong to (in the case of
 #' exact matching, coarsened exact matching, full matching, or
 #' subclassification). The formula for computing the weights depends on the
-#' argument supplied to `estimand`. A new stratum "propensity score"
-#' (`p`) is computed as the proportion of units in each stratum that are
+#' argument supplied to `estimand`. A new "stratum propensity score"
+#' (`sp`) is computed as the proportion of units in each stratum that are
 #' in the treated group, and all units in that stratum are assigned that
-#' propensity score. Weights are then computed using the standard formulas for
-#' inverse probability weights: for the ATT, weights are 1 for the treated
-#' units and `p/(1-p)` for the control units; for the ATC, weights are
-#' `(1-p)/p` for the treated units and 1 for the control units; for the
-#' ATE, weights are `1/p` for the treated units and `1/(1-p)` for the
+#' stratum propensity score. This is distinct from the propensity score used for matching, if any. Weights are then computed using the standard formulas for
+#' inverse probability weights with the stratum propensity score inserted: for the ATT, weights are 1 for the treated
+#' units and `sp/(1-sp)` for the control units; for the ATC, weights are
+#' `(1-sp)/sp` for the treated units and 1 for the control units; for the
+#' ATE, weights are `1/sp` for the treated units and `1/(1-sp)` for the
 #' control units. For cardinality matching, all matched units receive a weight
 #' of 1.
 #'
@@ -417,6 +409,7 @@ matchit <- function(formula,
   mcall <- match.call()
 
   ## Process method
+  .chk_null_or(method, chk::chk_string)
   if (length(method) == 1 && is.character(method)) {
     method <- tolower(method)
     method <- match_arg(method, c("exact", "cem", "nearest", "optimal", "full", "genetic", "subclass", "cardinality",
@@ -427,21 +420,19 @@ matchit <- function(formula,
     fn2 <- "matchit2null"
   }
   else {
-    stop("'method' must be the name of a supported matching method. See ?matchit for allowable options.", call. = FALSE)
+    .err("`method` must be the name of a supported matching method. See `?matchit` for allowable options")
   }
 
   #Process formula and data inputs
-  if (!inherits(formula, "formula")) stop("'formula' must be a formula object.", call. = FALSE)
+  .chk_formula(formula, sides = 2)
 
-  tt <- terms(formula, data = data)
-  if (attr(tt, "response") != 1) stop("A treatment variable must be included on the left-hand side of 'formula'.", call. = FALSE)
-  treat.form <- update(tt, . ~ 0)
+  treat.form <- update(terms(formula, data = data), . ~ 0)
   treat.mf <- model.frame(treat.form, data = data, na.action = "na.pass")
   treat <- model.response(treat.mf)
 
   #Check and binarize treat
   treat <- check_treat(treat)
-  if (length(treat) == 0) stop("The treatment cannot be NULL.", call. = FALSE)
+  if (length(treat) == 0) .err("the treatment cannot be `NULL`")
 
   names(treat) <- rownames(treat.mf)
 
@@ -467,28 +458,28 @@ matchit <- function(formula,
   if (!is.null(s.weights)) {
     if (is.character(s.weights)) {
       if (is.null(data) || !is.data.frame(data)) {
-        stop("If 's.weights' is specified a string, a data frame containing the named variable must be supplied to 'data'.", call. = FALSE)
+        .err("if `s.weights` is specified a string, a data frame containing the named variable must be supplied to `data`")
       }
       if (!all(s.weights %in% names(data))) {
-        stop("The name supplied to 's.weights' must be a variable in 'data'.", call. = FALSE)
+        .err("the name supplied to `s.weights` must be a variable in `data`")
       }
       s.weights.form <- reformulate(s.weights)
       s.weights <- model.frame(s.weights.form, data, na.action = "na.pass")
-      if (ncol(s.weights) != 1) stop("'s.weights' can only contain one named variable.", call. = FALSE)
+      if (ncol(s.weights) != 1) .err("`s.weights` can only contain one named variable")
       s.weights <- s.weights[[1]]
     }
     else if (inherits(s.weights, "formula")) {
-      s.weights.form <- update(s.weights, NULL ~ .)
+      s.weights.form <- update(terms(s.weights, data = data), NULL ~ .)
       s.weights <- model.frame(s.weights.form, data, na.action = "na.pass")
-      if (ncol(s.weights) != 1) stop("'s.weights' can only contain one named variable.", call. = FALSE)
+      if (ncol(s.weights) != 1) .err("`s.weights` can only contain one named variable")
       s.weights <- s.weights[[1]]
     }
     else if (!is.numeric(s.weights)) {
-      stop("'s.weights' must be supplied as a numeric vector, string, or one-sided formula.", call. = FALSE)
+      .err("`s.weights` must be supplied as a numeric vector, string, or one-sided formula")
     }
 
-    if (anyNA(s.weights)) stop("Missing values are not allowed in 's.weights'.", call. = FALSE)
-    if (length(s.weights) != n.obs) stop("'s.weights' must be the same length as the treatment vector.", call. = FALSE)
+    chk::chk_not_any_na(s.weights)
+    if (length(s.weights) != n.obs) .err("`s.weights` must be the same length as the treatment vector")
 
     names(s.weights) <- names(treat)
 
@@ -503,19 +494,21 @@ matchit <- function(formula,
     if (is.numeric(distance)) {
       fn1 <- "distance2user"
     }
-    else if (distance %in% matchit_distances()) {
-      fn1 <- "distance2mahalanobis"
-      is.full.mahalanobis <- TRUE
-      attr(is.full.mahalanobis, "transform") <- distance
-    }
-    else {
-      fn1 <- paste0("distance2", distance)
+    else if (is.character(distance)) {
+      if (distance %in% matchit_distances()) {
+        fn1 <- "distance2mahalanobis"
+        is.full.mahalanobis <- TRUE
+        attr(is.full.mahalanobis, "transform") <- distance
+      }
+      else {
+        fn1 <- paste0("distance2", distance)
+      }
     }
   }
 
   #Process covs
   if (!is.null(fn1) && fn1 == "distance2gam") {
-    check.package("mgcv")
+    rlang::check_installed("mgcv")
     env <- environment(formula)
     covs.formula <- mgcv::interpret.gam(formula)$fake.formula
     environment(covs.formula) <- env
@@ -524,6 +517,8 @@ matchit <- function(formula,
   else {
     covs.formula <- delete.response(terms(formula, data = data))
   }
+
+  covs.formula <- update(covs.formula, ~ .)
   covs <- model.frame(covs.formula, data = data, na.action = "na.pass")
   k <- ncol(covs)
   for (i in seq_len(k)) {
@@ -531,8 +526,8 @@ matchit <- function(formula,
       covariates.with.missingness <- names(covs)[i:k][vapply(i:k, function(j) anyNA(covs[[j]]) ||
                                                                (is.numeric(covs[[j]]) && any(!is.finite(covs[[j]]))),
                                                              logical(1L))]
-      stop(paste0("Missing and non-finite values are not allowed in the covariates. Covariates with missingness or non-finite values:\n\t",
-                  paste(covariates.with.missingness, collapse = ", ")), call. = FALSE)
+      .err(paste0("Missing and non-finite values are not allowed in the covariates. Covariates with missingness or non-finite values:\n\t",
+                  paste(covariates.with.missingness, collapse = ", ")), tidy = FALSE)
     }
     if (is.character(covs[[i]])) covs[[i]] <- factor(covs[[i]])
   }
@@ -620,7 +615,7 @@ matchit <- function(formula,
 
     if (!is.null(attr(caliper, "cal.formula"))) {
       calcovs <- model.frame(attr(caliper, "cal.formula"), data, na.action = "na.pass")
-      if (anyNA(calcovs)) stop("Missing values are not allowed in the covariates named in 'caliper'.", call. = FALSE)
+      if (anyNA(calcovs)) .err("missing values are not allowed in the covariates named in `caliper`")
       attr(caliper, "cal.formula") <- NULL
     }
   }
@@ -645,7 +640,7 @@ matchit <- function(formula,
   X.list <- list(covs, exactcovs, mahcovs, calcovs, antiexactcovs)
   all.covs <- lapply(X.list, names)
   for (i in seq_along(X.list)[-1]) if (!is.null(X.list[[i]])) X.list[[i]][names(X.list[[i]]) %in% unlist(all.covs[1:(i-1)])] <- NULL
-  X.list[lengths(X.list) == 0] <- NULL
+  X.list[vapply(X.list, is.null, logical(1L))] <- NULL
 
   ## putting all the results together
   out <- list(
@@ -669,10 +664,10 @@ matchit <- function(formula,
     obj = if (include.obj) match.out[["obj"]]
   )
 
-  out[lengths(out) == 0] <- NULL
+  out[vapply(out, is.null, logical(1L))] <- NULL
 
   class(out) <- class(match.out)
-  return(out)
+  out
 }
 
 #' @export
@@ -745,5 +740,5 @@ matchit2null <- function(discarded, ...) {
   res <- list(weights = as.numeric(!discarded))
   class(res) <- "matchit"
 
-  return(res)
+  res
 }
