@@ -70,9 +70,17 @@
 #' @param s.weights the variable containing sampling weights to be incorporated
 #' into propensity score models and balance statistics.
 #' @param replace whether matching should be done with replacement.
-#' @param m.order the order that the matching takes place. The default is
-#' `"largest"` when `distance` corresponds to a propensity score and
-#' `"data"` otherwise. See [matchit()] for allowable options.
+#' @param m.order the order that the matching takes place. Allowable options
+#'   include `"largest"`, where matching takes place in descending order of
+#'   distance measures; `"smallest"`, where matching takes place in ascending
+#'   order of distance measures; `"closest"`, where matching takes place in
+#'   order of the distance between units; `"random"`, where matching takes place
+#'   in a random order; and `"data"` where matching takes place based on the
+#'   order of units in the data. When `m.order = "random"`, results may differ
+#'   across different runs of the same code unless a seed is set and specified
+#'   with [set.seed()]. The default of `NULL` corresponds to `"largest"` when a
+#'   propensity score is estimated or supplied as a vector and `"data"`
+#'   otherwise.
 #' @param caliper the width(s) of the caliper(s) used for caliper matching. See
 #' Details and Examples.
 #' @param std.caliper `logical`; when calipers are specified, whether they
@@ -211,6 +219,10 @@
 #' `ratio`, and `max.controls` must be greater than `ratio`. See
 #' Examples below for an example of their use.
 #'
+#' ## Using `m.order = "closest"`
+#'
+#' As of version 4.6.0, `m.order` can be set to `"closest"`, which works regardless of how the distance measure is specified. This matches in order of the distance between units. The closest pair of units across all potential pairs of units will be matched first; the second closest pair of all potential pairs will be matched second, etc. This ensures that the best possible matches are given priority, and in that sense performs similarly to `m.order = "smallest"`.
+#'
 #' @seealso [matchit()] for a detailed explanation of the inputs and outputs of
 #' a call to `matchit()`.
 #'
@@ -266,15 +278,15 @@
 #'
 NULL
 
-matchit2nearest <-  function(treat, data, distance, discarded,
+matchit2nearest <- function(treat, data, distance, discarded,
                              ratio = 1, s.weights = NULL, replace = FALSE, m.order = NULL,
                              caliper = NULL, mahvars = NULL, exact = NULL,
                              formula = NULL, estimand = "ATT", verbose = FALSE,
-                             is.full.mahalanobis, fast = TRUE,
+                             is.full.mahalanobis,
                              antiexact = NULL, unit.id = NULL, ...){
 
   if (verbose) {
-    if (fast) check.package("RcppProgress")
+    rlang::check_installed("RcppProgress")
     cat("Nearest neighbor matching... \n")
   }
 
@@ -293,8 +305,8 @@ matchit2nearest <-  function(treat, data, distance, discarded,
 
   if (is.full.mahalanobis) {
     if (length(attr(terms(formula, data = data), "term.labels")) == 0) {
-      stop(sprintf("Covariates must be specified in the input formula when distance = \"%s\".",
-                   attr(is.full.mahalanobis, "transform")), call. = FALSE)
+      .err(sprintf("covariates must be specified in the input formula when `distance = \"%s\"`",
+                   attr(is.full.mahalanobis, "transform")))
     }
     mahvars <- formula
   }
@@ -323,6 +335,8 @@ matchit2nearest <-  function(treat, data, distance, discarded,
   else if (is.matrix(distance)) {
     distance_mat <- distance
     distance <- NULL
+
+    if (focal == 0) distance_mat <- t(distance_mat)
   }
 
   #Process caliper
@@ -381,7 +395,7 @@ matchit2nearest <-  function(treat, data, distance, discarded,
     ex <- factor(exactify(model.frame(exact, data = data), nam = lab, sep = ", ", include_vars = TRUE))
 
     cc <- intersect(as.integer(ex)[treat==1], as.integer(ex)[treat==0])
-    if (length(cc) == 0) stop("No matches were found.", call. = FALSE)
+    if (length(cc) == 0) .err("No matches were found")
 
     if (reuse.max < n1) {
 
@@ -391,16 +405,16 @@ matchit2nearest <-  function(treat, data, distance, discarded,
       }, numeric(1L))
 
       if (any(e_ratios < 1)) {
-        warning(sprintf("Fewer %s units than %s units in some 'exact' strata; not all %s units will get a match.",
-                        tc[2], tc[1], tc[1]), immediate. = TRUE, call. = FALSE)
+        .wrn(sprintf("fewer %s units than %s units in some `exact` strata; not all %s units will get a match",
+                        tc[2], tc[1], tc[1]))
       }
       if (ratio > 1 && any(e_ratios < ratio)) {
         if (is.null(max.controls) || ratio == max.controls)
-          warning(sprintf("Not all %s units will get %s matches.",
-                          tc[1], ratio), immediate. = TRUE, call. = FALSE)
+          .wrn(sprintf("not all %s units will get %s matches",
+                          tc[1], ratio))
         else
-          warning(sprintf("Not enough %s units for an average of %s matches per %s unit in all 'exact' strata.",
-                          tc[2], ratio, tc[1]), immediate. = TRUE, call. = FALSE)
+          .wrn(sprintf("not enough %s units for an average of %s matches per %s unit in all `exact` strata",
+                          tc[2], ratio, tc[1]))
       }
     }
   }
@@ -415,19 +429,16 @@ matchit2nearest <-  function(treat, data, distance, discarded,
       }
 
       if (e_ratios < 1) {
-        warning(sprintf("Fewer %s %s than %s units; not all %s units will get a match.",
-                        tc[2], if (is.null(unit.id)) "units" else "unit IDs", tc[1], tc[1]),
-                immediate. = TRUE, call. = FALSE)
+        .wrn(sprintf("fewer %s %s than %s units; not all %s units will get a match",
+                        tc[2], if (is.null(unit.id)) "units" else "unit IDs", tc[1], tc[1]))
       }
       else if (e_ratios < ratio) {
         if (is.null(max.controls) || ratio == max.controls)
-          warning(sprintf("Not all %s units will get %s matches.",
-                          tc[1], ratio),
-                  immediate. = TRUE, call. = FALSE)
+          .wrn(sprintf("not all %s units will get %s matches",
+                          tc[1], ratio))
         else
-          warning(sprintf("Not enough %s %s for an average of %s matches per %s unit.",
-                          tc[2], if (is.null(unit.id)) "units" else "unit IDs", ratio, tc[1]),
-                  immediate. = TRUE, call. = FALSE)
+          .wrn(sprintf("not enough %s %s for an average of %s matches per %s unit",
+                          tc[2], if (is.null(unit.id)) "units" else "unit IDs", ratio, tc[1]))
       }
     }
   }
@@ -436,13 +447,13 @@ matchit2nearest <-  function(treat, data, distance, discarded,
   #Each treated unit get its own value of ratio
   if (!is.null(max.controls)) {
     if (is.null(distance)) {
-      if (is.full.mahalanobis) stop(sprintf("'distance' cannot be \"%s\" for variable ratio matching.",
-                                            transform), call. = FALSE)
-      else stop("'distance' cannot be supplied as a matrix for variable ratio matching.", call. = FALSE)
+      if (is.full.mahalanobis) .err(sprintf("`distance` cannot be \"%s\" for variable ratio matching",
+                                            transform))
+      .err("`distance` cannot be supplied as a matrix for variable ratio matching")
     }
 
     m <- round(ratio * n1)
-    # if (m > sum(treat == 0)) stop("'ratio' must be less than or equal to n0/n1.", call. = FALSE)
+    # if (m > sum(treat == 0)) stop("'ratio' must be less than or equal to n0/n1")
 
     kmax <- floor((m - min.controls*(n1-1)) / (max.controls - min.controls))
     kmin <- n1 - kmax - 1
@@ -469,33 +480,56 @@ matchit2nearest <-  function(treat, data, distance, discarded,
   }
 
   m.order <- {
-    if (is.null(distance)) match_arg(m.order, c("data", "random"))
-    else if (!is.null(m.order)) match_arg(m.order, c("largest", "smallest", "random", "data"))
+    if (is.null(distance)) match_arg(m.order, c("data", "random", "closest"))
+    else if (!is.null(m.order)) match_arg(m.order, c("largest", "smallest", "data", "random", "closest"))
     else if (estimand == "ATC") "smallest"
     else "largest"
   }
 
   if (is.null(ex) || !is.null(unit.id)) {
-    ord <- switch(m.order,
-                  "largest" = order(distance[treat == 1], decreasing = TRUE),
-                  "smallest" = order(distance[treat == 1], decreasing = FALSE),
-                  "random" = sample.int(n1),
-                  "data" = seq_len(n1))
+    if (m.order == "closest") {
+      if (!is.null(mahcovs)) {
+        distance_mat <- eucdist_internal(mahcovs, treat)
+
+        # If caliper on PS (distance), treat it as a covariate
+        if (!is.null(distance) && !is.null(caliper.dist)) {
+          caliper.covs.mat <- {
+            if (is.null(caliper.covs)) as.matrix(distance)
+            else cbind(caliper.covs.mat, distance)
+          }
+          caliper.covs <- c(caliper.covs, caliper.dist)
+          caliper.dist <- NULL
+        }
+      }
+      else if (is.null(distance_mat)) {
+        distance_mat <- eucdist_internal(distance, treat)
+      }
+
+      ord <- NULL
+    }
+    else {
+      ord <- switch(m.order,
+                    "largest" = order(distance[treat == 1], decreasing = TRUE),
+                    "smallest" = order(distance[treat == 1], decreasing = FALSE),
+                    "random" = sample.int(n1),
+                    "data" = seq_len(n1))
+    }
 
     mm <- nn_matchC_dispatch(treat, ord, ratio, discarded, reuse.max, distance, distance_mat,
                              ex, caliper.dist, caliper.covs, caliper.covs.mat, mahcovs,
-                             antiexactcovs, unit.id, verbose)
+                             antiexactcovs, unit.id, m.order, verbose)
 
   }
   else {
     distance_ <- caliper.covs.mat_ <- mahcovs_ <- antiexactcovs_ <- distance_mat_ <- NULL
-    mm_list <- lapply(levels(ex), function(e) {
+    mm_list <- lapply(levels(ex)[cc], function(e) {
       if (verbose) {
         cat(sprintf("Matching subgroup %s/%s: %s...\n",
-                    match(e, levels(ex)), nlevels(ex), e))
+                    match(e, levels(ex)[cc]), length(cc), e))
       }
 
       .e <- which(ex == e)
+      .e1 <- which(ex[treat==1] == e)
       treat_ <- treat[.e]
       discarded_ <- discarded[.e]
       if (!is.null(distance)) distance_ <- distance[.e]
@@ -503,29 +537,49 @@ matchit2nearest <-  function(treat, data, distance, discarded,
       if (!is.null(mahcovs)) mahcovs_ <- mahcovs[.e,,drop = FALSE]
       if (!is.null(antiexactcovs)) antiexactcovs_ <- antiexactcovs[.e,,drop = FALSE]
       if (!is.null(distance_mat)) {
-        .e1 <- which(ex[treat==1] == e)
         .e0 <- which(ex[treat==0] == e)
         distance_mat_ <- distance_mat[.e1, .e0, drop = FALSE]
       }
-      ratio_ <- ratio[ex[treat==1]==e]
+      ratio_ <- ratio[.e1]
 
       n1_ <- sum(treat_ == 1)
-      ord_ <- switch(m.order,
-                     "largest" = order(distance_[treat_ == 1], decreasing = TRUE),
-                     "smallest" = order(distance_[treat_ == 1], decreasing = FALSE),
-                     "random" = sample.int(n1_),
-                     "data" = seq_len(n1_))
+
+
+      if (m.order == "closest") {
+        if (!is.null(mahcovs)) {
+          distance_mat_ <- eucdist_internal(mahcovs_, treat_)
+        }
+        else if (is.null(distance_mat)) {
+          distance_mat_ <- eucdist_internal(distance_, treat_)
+        }
+
+        ord <- NULL
+      }
+      else {
+        ord_ <- switch(m.order,
+                       "largest" = order(distance_[treat_ == 1], decreasing = TRUE),
+                       "smallest" = order(distance_[treat_ == 1], decreasing = FALSE),
+                       "random" = sample.int(n1_),
+                       "data" = seq_len(n1_))
+      }
 
       mm_ <- nn_matchC_dispatch(treat_, ord_, ratio_, discarded_, reuse.max, distance_, distance_mat_,
                                NULL, caliper.dist, caliper.covs, caliper.covs.mat_, mahcovs_,
-                               antiexactcovs_, NULL, verbose)
+                               antiexactcovs_, NULL, m.order, verbose)
 
       #Ensure matched indices correspond to indices in full sample, not subgroup
       mm_[] <- seq_along(treat)[.e][mm_]
       mm_
     })
 
-    mm <- do.call("rbind", mm_list)[lab1,, drop = FALSE]
+    #Construct match.matrix
+    mm <- matrix(NA_integer_, nrow = length(lab1),
+                 ncol = max(vapply(mm_list, ncol, numeric(1L))),
+                 dimnames = list(lab1, NULL))
+
+    for (m in mm_list) {
+      mm[rownames(m), seq_len(ncol(m))] <- m
+    }
   }
 
   if (verbose) cat("Calculating matching weights... ")
@@ -539,21 +593,26 @@ matchit2nearest <-  function(treat, data, distance, discarded,
 
   res <- list(match.matrix = nummm2charmm(mm, treat),
               subclass = psclass,
-              weights = weights.matrix(mm, treat))
+              weights = get_weights_from_mm(mm, treat))
 
   if (verbose) cat("Done.\n")
 
   class(res) <- "matchit"
 
-  return(res)
+  res
 }
 
 # Dispatches Rcpp function for NN matching
 # nn_matchC_vec() if distance_mat and mahcovs are NULL
 # nn_matchC() otherwise
 nn_matchC_dispatch <- function(treat, ord, ratio, discarded, reuse.max, distance, distance_mat, ex, caliper.dist,
-                               caliper.covs, caliper.covs.mat, mahcovs, antiexactcovs, unit.id, verbose) {
-  if (is.null(distance_mat) && is.null(mahcovs)) {
+                               caliper.covs, caliper.covs.mat, mahcovs, antiexactcovs, unit.id, m.order, verbose) {
+  if (m.order == "closest") {
+    nn_matchC_closest(distance_mat, treat, ratio, discarded, reuse.max,
+                      ex, caliper.dist, caliper.covs, caliper.covs.mat,
+                      antiexactcovs, unit.id, verbose)
+  }
+  else if (is.null(distance_mat) && is.null(mahcovs)) {
     nn_matchC_vec(treat, ord, ratio, discarded, reuse.max, distance,
                   ex, caliper.dist, caliper.covs, caliper.covs.mat,
                   antiexactcovs, unit.id, verbose)
@@ -563,4 +622,4 @@ nn_matchC_dispatch <- function(treat, ord, ratio, discarded, reuse.max, distance
               ex, caliper.dist, caliper.covs, caliper.covs.mat, mahcovs,
               antiexactcovs, unit.id, verbose)
   }
-}
\ No newline at end of file
+}
