@@ -246,7 +246,7 @@ matchit2optimal <- function(treat, formula, data, distance, discarded,
                             estimand = "ATT", verbose = FALSE,
                             is.full.mahalanobis,  antiexact = NULL, ...) {
 
-  check.package("optmatch")
+  rlang::check_installed("optmatch")
 
   if (verbose) cat("Optimal matching... \n")
 
@@ -276,14 +276,14 @@ matchit2optimal <- function(treat, formula, data, distance, discarded,
 
   if (is.full.mahalanobis) {
     if (length(attr(terms(formula, data = data), "term.labels")) == 0) {
-      stop(sprintf("Covariates must be specified in the input formula when distance = \"%s\".",
-                   attr(is.full.mahalanobis, "transform")), call. = FALSE)
+      .err(sprintf("covariates must be specified in the input formula when `distance = \"%s\"`",
+                   attr(is.full.mahalanobis, "transform")))
     }
     mahvars <- formula
   }
 
   if (!is.null(caliper)) {
-    warning("Calipers are currently not compatible with method = \"optimal\" and will be ignored.", call. = FALSE, immediate. = TRUE)
+    .wrn("calipers are currently not compatible with `method = \"optimal\"` and will be ignored")
     caliper <- NULL
   }
 
@@ -299,39 +299,41 @@ matchit2optimal <- function(treat, formula, data, distance, discarded,
     ex <- factor(exactify(model.frame(exact, data = data),
                           sep = ", ", include_vars = TRUE)[!discarded])
 
-    cc <- intersect(ex[treat_==1], ex[treat_==0])
-    if (length(cc) == 0) stop("No matches were found.", call. = FALSE)
+    cc <- intersect(as.integer(ex)[treat_==1], as.integer(ex)[treat_==0])
+    if (length(cc) == 0) .err("No matches were found")
 
     e_ratios <- vapply(levels(ex), function(e) sum(treat_[ex == e] == 0)/sum(treat_[ex == e] == 1), numeric(1L))
 
     if (any(e_ratios < 1)) {
-      warning(sprintf("Fewer %s units than %s units in some 'exact' strata; not all %s units will get a match.",
-                      tc[2], tc[1], tc[1]), immediate. = TRUE, call. = FALSE)
+      .wrn(sprintf("Fewer %s units than %s units in some `exact` strata; not all %s units will get a match",
+                      tc[2], tc[1], tc[1]))
     }
     if (ratio > 1 && any(e_ratios < ratio)) {
       if (ratio == max.controls)
-        warning(sprintf("Not all %s units will get %s matches.",
-                        tc[1], ratio), immediate. = TRUE, call. = FALSE)
+        .wrn(sprintf("Not all %s units will get %s matches",
+                        tc[1], ratio))
       else
-        warning(sprintf("Not enough %s units for an average of %s matches per %s unit in all 'exact' strata.",
-                        tc[2], ratio, tc[1]), immediate. = TRUE, call. = FALSE)
+        .wrn(sprintf("Not enough %s units for an average of %s matches per %s unit in all `exact` strata",
+                        tc[2], ratio, tc[1]))
     }
   }
   else {
     ex <- factor(rep("_", length(treat_)), levels = "_")
+    cc <- 1
+
     e_ratios <- setNames(sum(treat_ == 0)/sum(treat_ == 1), levels(ex))
 
     if (e_ratios < 1) {
-      warning(sprintf("Fewer %s units than %s units; not all %s units will get a match.",
-                      tc[2], tc[1], tc[1]), immediate. = TRUE, call. = FALSE)
+      .wrn(sprintf("Fewer %s units than %s units; not all %s units will get a match",
+                      tc[2], tc[1], tc[1]))
     }
     else if (e_ratios < ratio) {
       if (ratio == max.controls)
-        warning(sprintf("Not all %s units will get %s matches.",
-                        tc[1], ratio), immediate. = TRUE, call. = FALSE)
+        .wrn(sprintf("Not all %s units will get %s matches",
+                        tc[1], ratio))
       else
-        warning(sprintf("Not enough %s units for an average of %s matches per %s unit.",
-                        tc[2], ratio, tc[1]), immediate. = TRUE, call. = FALSE)
+        .wrn(sprintf("Not enough %s units for an average of %s matches per %s unit",
+                        tc[2], ratio, tc[1]))
     }
   }
 
@@ -348,7 +350,7 @@ matchit2optimal <- function(treat, formula, data, distance, discarded,
     mo <- distance
   }
   else {
-    mo <- eucdist_internal(setNames(distance, names(treat)), treat)
+    mo <- eucdist_internal(setNames(as.numeric(distance), names(treat)), treat)
   }
 
   #Transpose distance mat as needed
@@ -373,16 +375,20 @@ matchit2optimal <- function(treat, formula, data, distance, discarded,
   pair <- setNames(rep(NA_character_, length(treat)), names(treat))
   p <- setNames(vector("list", nlevels(ex)), levels(ex))
 
-  t_df <- data.frame(treat)
+  t_df <- data.frame(treat_)
 
-  for (e in levels(ex)) {
+  for (e in levels(ex)[cc]) {
     if (nlevels(ex) > 1) {
+      if (verbose) {
+        cat(sprintf("Matching subgroup %s/%s: %s...\n",
+                    match(e, levels(ex)[cc]), length(cc), e))
+      }
       mo_ <- mo[ex[treat_==1] == e, ex[treat_==0] == e]
     }
     else mo_ <- mo
 
-    if (any(dim(mo_) == 0)) next
-    else if (all(dim(mo_) == 1)) {
+    if (any(dim(mo_) == 0) || !any(is.finite(mo_))) next
+    else if (all(dim(mo_) == 1) && all(is.finite(mo_))) {
       pair[ex == e] <- paste(1, e, sep = "|")
       next
     }
@@ -406,27 +412,20 @@ matchit2optimal <- function(treat, formula, data, distance, discarded,
       max.controls_ <- max.controls
     }
 
-    withCallingHandlers({
+    matchit_try({
       p[[e]] <- do.call(optmatch::fullmatch,
                         c(list(mo_,
                                mean.controls = ratio_,
                                min.controls = min.controls_,
                                max.controls = max.controls_,
-                               data = t_df), #just to get rownames; not actually used in matching
+                               data = t_df[ex == e,, drop = FALSE]), #just to get rownames; not actually used in matching
                           A))
-    },
-    warning = function(w) {
-      warning(paste0("(from optmatch) ", conditionMessage(w)), call. = FALSE, immediate. = TRUE)
-      invokeRestart("muffleWarning")
-    },
-    error = function(e1) {
-      stop(paste0("(from optmatch) ", conditionMessage(e1)), call. = FALSE)
-    })
+    }, from = "optmatch")
 
     pair[names(p[[e]])[!is.na(p[[e]])]] <- paste(as.character(p[[e]][!is.na(p[[e]])]), e, sep = "|")
   }
 
-  if (all(is.na(pair))) stop("No matches were found.", call. = FALSE)
+  if (all(is.na(pair))) .err("No matches were found")
   if (length(p) == 1) p <- p[[1]]
 
   psclass <- factor(pair)
@@ -440,11 +439,11 @@ matchit2optimal <- function(treat, formula, data, distance, discarded,
   ## calculate weights and return the results
   res <- list(match.matrix = mm,
               subclass = psclass,
-              weights = weights.subclass(psclass, treat, estimand),
+              weights = get_weights_from_subclass(psclass, treat, estimand),
               obj = p)
 
   if (verbose) cat("Done.\n")
 
   class(res) <- "matchit"
-  return(res)
+  res
 }
